@@ -9,7 +9,7 @@
 #include <stdint.h>
 
 static inline int
-compute_sampled_lines (int height, int y_sampling, int start_y)
+compute_sampled_height (int height, int y_sampling, int start_y)
 {
     int nlines;
 
@@ -31,7 +31,7 @@ compute_sampled_lines (int height, int y_sampling, int start_y)
         else
             start = start_y;
         end = start_y + height - 1;
-        end -= (end % y_sampling);
+        end -= (end < 0) ? (-end % y_sampling) : (end % y_sampling);
 
         if (start > end)
             nlines = 0;
@@ -42,4 +42,19 @@ compute_sampled_lines (int height, int y_sampling, int start_y)
     return nlines;
 }
 
+static inline int
+compute_sampled_width (int width, int x_sampling, int start_x)
+{
+    /*
+     * we require that the start_x % x_sampling == 0 and for tiled images (and for deep),
+     * x_sampling must be 1, so this can simplify the math compared to the y case
+     * where when we are reading scanline images, we always are reading the entire
+     * width. If this changes, can look like the above call for the lines, but
+     * for now can be simpler math
+     */
+    if (x_sampling <= 1) return width;
+
+    return (width == 1) ? 1 : (width / x_sampling);
+}
+
 #endif /* OPENEXR_PRIVATE_UTIL_H */
